<?php

declare(strict_types=1);

namespace Drupal\Tests\drupal_cms_admin_theme\Functional;

use Drupal\FunctionalTests\Core\Recipe\RecipeTestTrait;
use Drupal\Tests\BrowserTestBase;

/**
 * @group drupal_cms_admin_ui
 */
class ComponentValidationTest extends BrowserTestBase {

  use RecipeTestTrait;

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  protected static $modules = ['block'];

  public function test(): void {
    $dir = realpath(__DIR__ . '/../../..');

    // The recipe should apply cleanly.
    $this->applyRecipe($dir);
    // Apply it again to prove that it is idempotent.
    $this->applyRecipe($dir);

    $account = $this->drupalCreateUser([
      'access navigation',
      'view welcome dashboard',
      // Needed to see the announcements block.
      'access announcements',
      // Permissions needed to access the top tasks.
      'administer menu',
      'administer modules',
      'administer themes',
      'administer users',
    ]);
    // Don't use one-time login links, because they will bypass the dashboard
    // upon login.
    $this->useOneTimeLoginLinks = FALSE;
    $this->drupalLogin($account);
    $assert_session = $this->assertSession();
    // The Help module is not installed, so a link to it should not be present
    // in the navigation.
    $footer = $assert_session->elementExists('css', 'nav > h3:contains("Administrative toolbar footer")')
      ->getParent();
    $assert_session->elementNotExists('named', ['link', 'Help'], $footer);

    // The navigation should have a link to the dashboard.
    $assert_session->elementAttributeContains('named', ['link', 'Dashboard'], 'class', 'toolbar-button--icon--navigation-dashboard');

    // We should be on the welcome dashboard.
    $assert_session->addressEquals('/admin/dashboard');
    // There should be a menu of top tasks, and everything in it should be
    // accessible to us.
    $top_tasks = $assert_session->elementExists('css', 'h2:contains("Top tasks") + ul');
    $top_task_links = [
      'Choose recommended add-ons',
      'Browse modules',
      'Change site appearance',
      'Invite users to collaborate',
      'Edit top tasks',
    ];
    foreach ($top_task_links as $link) {
      $top_tasks->clickLink($link);
      $this->assertSame(200, $this->getSession()->getStatusCode(), "The '$link' link was not accessible.");
      $this->drupalGet('/admin/dashboard');
    }
    // We should see the Announcements block.
    $assert_session->pageTextContains('Announcements');
    $assert_session->elementExists('named', ['link', 'View all announcements']);

    // Ensure that the Project Browser tabs are renamed as expected.
    $this->drupalPlaceBlock('local_tasks_block');
    $this->drupalGet('/admin/modules');
    // Get the Project Browser local tasks.
    $elements = $assert_session->elementExists('css', 'h2:contains("Primary tabs")')
      ->getParent()
      ->findAll('css', 'a');
    $local_tasks = [];
    foreach ($elements as $element) {
      $link_text = $element->getText();
      $local_tasks[$link_text] = $element->getAttribute('data-drupal-link-system-path');
    }
    // The first task should go to core's regular modules page.
    $this->assertSame('admin/modules', reset($local_tasks));
    // Ensure the Project Browser tasks are in the expected order, have the
    // expected link text, and link to the expected place.
    $project_browser_tasks = preg_grep('|admin/modules/browse/.+|', $local_tasks);
    $this->assertSame(['Recommended', 'Browse modules'], array_keys($project_browser_tasks));
    $this->assertStringEndsWith('/recipes', $project_browser_tasks['Recommended']);
    $this->assertStringEndsWith('/drupalorg_jsonapi', $project_browser_tasks['Browse modules']);
    // We should have access to all Project Browser tasks.
    foreach ($project_browser_tasks as $path) {
      $this->drupalGet($path);
      $assert_session->statusCodeEquals(200);
    }

    $this->drupalLogout();
    // Ensure that there are no broken blocks in the navigation (or anywhere
    // else). We need to test this with the root user because they have all
    // permissions, and therefore any broken blocks in the navigation will be
    // obvious to them.
    $this->drupalLogin($this->rootUser);
    $assert_session->pageTextNotContains('This block is broken or missing.');
  }

}
